﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary; 


namespace PlanetaryDomination
{
    [Serializable()] 
    public class Sector
    {
        public int X, Y, Number, Owner, Ore, Anorganics, Energy, BuildBuilding, BuildUnitID, UnitBuildCost, AttackersCount;
        public List<bool> Examined; // array mit bool für jeden spieler das besagt ob der sektor bereits untersucht wurde (prospektor, nur einmal nötig), startsectoren sind automatisch untersucht
        public List<int> Scanning; // array mit int für jeden spieler das besagt mit welcher stärke der sektor gescannt wird,
        public static int NumberofPlayers;
        public List<Unit> Units;
        public List<Building> Buildings;
        public bool Moveable, Attackable, BuildNoMore;

        public Sector(int x, int y, int number)
        {
            X = x;
            Y = y;
            Number = number;
            BuildBuilding = BuildUnitID = -1;
            UnitBuildCost = AttackersCount = 0;  // bestimmt wie viel der einheit bereits gebaut wurde (fabriklvl wird raufaddiert) und wenn >= baukosten wird einheit fertig, das bezieht sich auf production cost, die costen für resourcen werden am anfang einmalig abgezogen
            Owner = 8;  // 0-7 die spieler und 8 für unbesetzt
            Moveable = BuildNoMore = Attackable = false;
            Examined = new List<bool>();
            for (int i = 0; i < NumberofPlayers; i++)
            {
                Examined.Add(false);    // alle sectoren sind zu begin unerfroscht, der eigene wird bei der belegung gesetzt
            }
            Units = new List<Unit>();
            Buildings = new List<Building>();
            Buildings.Add(new Building("Mine"));    // #0
            Buildings.Add(new Building("Greenhouse"));    // #1
            Buildings.Add(new Building("Powerplant"));    // #2
            Buildings.Add(new Building("Factory"));    // #3
            Buildings.Add(new Building("Barracks"));    // #4
            Buildings.Add(new Building("Academy"));    // #5
            Buildings.Add(new Building("Scanner"));    // #6
            Buildings.Add(new Building("Transport"));    // #7
        }
        public void ProcessTurn()
        {// berechnet die runde (bewegt einheiten?, baut gebäude und einheiten)
            if ((-1 != BuildBuilding) && (false == BuildNoMore))
            {// es wird ein gebäude gebaut in diesem sektor also den bau fortschreiten lassen
                if (Buildings[BuildBuilding].BuildTurn())   // wenn gebäude fertig gebaut auch zurücksetzen damit man neues wählen muss
                    BuildBuilding = -1;
            }
            for (int i = 0; i < Units.Count; i++)
            {
                if (Units[i].ETA > 0)   // die einheit ist in bewegung also näherkommen lassen
                    --Units[i].ETA;
            }
            AttackersCount = 0; // falls sektor von spieler angegriffen wurde das jede runde zurück setzen da kämpfe entschieden sind
        }
        public bool FightOccurs()
        {// prüft ob es diese runde zu einem kampf kommt
            int i,j;
            if (Owner != 8)
            {// sektor gehört jemandem also prüfen ob fremde einheiten da sind
                for (i = 0; i < Units.Count; i++)
                {
                    if (Units[i].Owner != Owner)
                        return true;
                }
            }
            else if (Units.Count >1)
            {// sektor ist unbewohnt und mehr als zwei einheiten da also prüfen ob einheiten von verschiedenen besitzern da sind
                for (i = 0; i < Units.Count; i++)
                {// für jede einheit durchgehen und prüfen ob eine andere einheit von anderem owner da ist (eta = 0)
                    for (j = i; j < Units.Count; j++)
                    {
                        if ((i != j) && (Units[i].ETA == 0) && (Units[j].ETA == 0) && (Units[i].Owner != Units[j].Owner))
                            return true;
                    }
                }
            }
            return false;   // wenn ich noch hier bin gibts keinen kampf
        }
        public int GetUnitOwners()
        {// gibt die anzahl verschiedener besitzer der truppen in diesem sektor zurück
            int count = 0, i;
            for (i = 0; i < 8; i++)
            {
                if(GetUnitCount(i) > 0)
                {
                    ++count;
                }
            }
            return count;
        }
        public int GetUnitCount(int owner)
        {// bestimmt die anzahl der einheiten eines bestimmten besitzers in dem sektor
            int count = 0, i;
            for (i = 0; i < Units.Count; i++)
            {
                if(owner == Units[i].Owner)
                {
                    ++count;
                }
            }
            return count;
        }
    }
}
